import requests
import json
from loguru import logger


class Provider:
    def __init__(self, stack: str, username: str, password: str):
        self.username = username
        self.password = password
        self.stack = stack
        self.endpoint = f"{self.stack}.datatap.adverity.com"

    def get_token(self):
        payload = {
            'username': self.username,
            'password': self.password
        }
        headers = {'content-type': 'application/json'}
        try:
            r = requests.post(f"https://{self.endpoint}/api/auth/token/",
                              data=json.dumps(payload), headers=headers)
            token = json.loads(r.text)['token']
        except:
            logger.exception("Failed to get login token")
        return token


if __name__ == '__main__':
    username = 'rohan+api@rubixagency.com'
    password = 'AeUDm6Bp7d2jqJE6'
    stack = 'rubix'
    p = Provider(stack=stack, username=username, password=password)
    print(p.get_token())



from typing import List
from provider import Provider
import requests
import json
import os
from loguru import logger


def pagination(url: str, token: str) -> List:
    headers = {
        'Authorization': f"Token {token}",
        'Content-Type': 'application/json'
    }
    results = []
    next_page = url
    while next_page:
        r = requests.get(next_page, headers=headers)
        response = json.loads(r.text)
        next_page = response['next']
        results.extend(response['results'])
    return results


class Connection:
    def __init__(self, stack, connection_type_id, connection_id, connection_name):
        self.stack = stack
        self.connection_type_id = connection_type_id
        self.connection_id = connection_id
        self.connection_name = connection_name


class ConnectionFactory:
    def __init__(self, provider):
        self.provider = provider

    def get_connection_types(self):
        """Retrieve all white-listed Connection types on any particular stack.

        The result is paginated

        https://help.adverity.com/hc/en-us/articles/360011404100-Connections-Connection-Types
        curl -X GET https://ENDPOINT/api/connection-types/ -H "Authorization: Token TOKEN_VALUE" 
        """
        try:
            results = pagination(f"https://{self.provider.endpoint}/api/connection-types/", self.provider.get_token())
        except:
            logger.exception("Failed to get connection types")
        return results

    def retrieve_connections(self, connection_type_id: str):
        """Use the Connection type ID, e.g. 123 to retrieve all connections of this type.

        curl -H https://ENDPOINT/api/connection-types/123/connections/ -H "Authorization: Token TOKEN_VALUE"

        :param connection_type_id: connection type id
        :type connection_type_id: str
        """
        try:
            results = pagination(
                f"https://{self.provider.endpoint}/api/connection-types/{connection_type_id}/connections/", self.provider.get_token())
        except:
            logger.exception(f"Failed to retrieve connections of type {connection_type_id}")
        return results

    def create_connection(self, connection_type_id: str, stack_id: int, name: str, **args) -> Connection:
        """Create Connection

        make an OPTIONS request against the Connection type of your choice to find out about available/mandatory options
        curl -X OPTIONS https://YOUR_STACK.datatap.adverity.com/api/connection-types/123/connections/ -H 'Authorization: Token TOKEN_VALUE'

        curl -X POST \
        https://YOUR_STACK.datatap.adverity.com/api/connection-types/123/connections/ \
        -H 'Authorization: Token TOKEN_VALUE' \
        -H 'Content-Type: application/json' \
        -d '{
        "datatype": "DATATYPE",
        "cron_type": "CRON_TYPE",
        "name": "NAME_OF_STREAM",
        "stack": WORKSPACE_ID
        }'
        """
        try:
            headers = {
                'Authorization': f"Token {self.provider.get_token()}",
                'Content-Type': 'application/json'
            }
            url = f"https://{self.provider.stack}.datatap.adverity.com/api/connection-types/{connection_type_id}/connections/"
            if connection_type_id == '739':
                # create connection for type 739, Facebook Ads Connection
                # name, required
                # no_validate, optional
                # stack, optional
                data = {
                    'name': name,
                    'stack': stack_id
                }
                response = requests.post(url, json=data, headers=headers)
                logger.info(response.text)
            else:
                raise Exception(f"Connection type {connection_type_id} is not implemented")
        except:
            logger.exception(f"Failed to create connection for connection_type_id {connection_type_id}")
            raise
        else:
            # check if the connection is already created or not
            try:
                return Connection(1, connection_type_id, json.loads(response.text)['id'], json.loads(response.text)['name'])
            except:
                logger.exception(f"This connection name already exists")
                raise Exception(f"This connection name already exists")

    def update_connection_token(self, connection: Connection, new_token: str):
        """Update token of the connection

        curl -X PATCH \
        https://ENDPOINT/api/connection-types/CONNECTION_TYPE_ID/connections/CONNECTION_ID/token/
        -H "Authorization: Token TOKEN_VALUE" \
        -H 'Content-Type: application/json' \
        -d access_token=SYSTEM_GENERATED_TOKEN \

        :param connection: adverity connection
        :type connection: Connection
        :param new_token: new connection token
        :type new_token: str
        """
        try:
            headers = {
                'Authorization': f"Token {self.provider.get_token()}",
                'Content-Type': 'application/json'
            }
            connection_id = connection.connection_id
            connection_type_id = connection.connection_type_id
            response = requests.patch(
                f"https://{self.provider.endpoint}/api/connection-types/{connection_type_id}/connections/{connection_id}/token/",
                headers=headers,
                json={'access_token': new_token, 'expires': '2100-01-01T01:01'}
            )
            logger.info(response.text)
        except:
            logger.exception(f"Failed to update token on connection type {connection_type_id} id {connection_id}")


if __name__ == '__main__':
    username = os.getenv('adverity_username')
    password = os.getenv('adverity_password')

    # stack_id 1
    stack = 'rubix'
    p = Provider(stack=stack, username=username, password=password)

    # create connection factory
    connection_factory = ConnectionFactory(provider=p)

    # get all connection types
    results = connection_factory.get_connection_types()
    for result in results:
        if result['id'] == 739:
            print(result)

    # get all connections for a certain connection type
    results = connection_factory.retrieve_connections('739')
    print(results)

    # create connection
    try:
        connection_factory.create_connection('739', 1,  'act_319883982')
    except:
        pass

    # update connection token
    test_connection = Connection(1, '739', 21, 'test1')
    new_token = 'EAAhMVyE1D1kBAOVnBJadfloQJiuVZAzfzHE4jnYfMbKYNBrH1Iu3OZBXWfXsIZAtI1fVZAYCwqyOoxQpDKkcW0jdULTZAyGbH3fu5oUaMs7i6PXoo0yEiEo9G3e1QaRTEDCZBrLkxMqpC7tFmXE364M1jzV4KgwOKQWpOqcb4YCNaacwhJxs9a'
    connection_factory.update_connection_token(test_connection, new_token)



import logging
import boto3
from sqlalchemy import create_engine
from .helper import get_value_from_ssm
import requests
import json
from loguru import logger

# AWS Credential
AWS_REGION = 'us-east-1'
# Channel Analytics
ANALYTICS_TOKEN_URI = 'https://accounts.google.com/o/oauth2/token'
SHOPIFY_API_VERSION = '2021-01'


class SecretsManager:
    client = boto3.client('ssm', region_name=AWS_REGION)

    # refactor this into a connection object
    def __init__(self, host: str, port: str, username: str, password: str, database: str):
        self.connection_string = self._create_connection_string(host, port, username, password, database)
        self.engine = create_engine(self.connection_string)
        self.conn = None

    def _create_connection_string(self, host: str, port: str, username: str, password: str, database: str):
        return f"postgres://{username}:{password}@{host}:{port}/{database}"

    def _get_connection(self):
        """Create a singleton connection object."""
        if self.conn is None:
            self.conn = self.engine.connect()
        return self.conn

    def get_facebook_act_ids(self):
        channel = 'facebook'
        sql_statement = f"""
            -- choose the latest if multiple apply
            SELECT distinct ad_object_id
            FROM {channel}_integration
        """
        logger.info("Fetching all act_ids for channel facebook")
        logger.debug(sql_statement)
        results = self._get_connection().execute(sql_statement)
        act_ids = [result[0] for result in results]
        return act_ids

    def get_facebook_access_token(self, id: str):
        if not id.startswith("act_"):
            act_id = f"act_{id}"
        else:
            act_id = id

        channel = "facebook"
        sql_statement = f"""
            -- choose the latest if multiple apply
            SELECT access_token
            FROM {channel}_integration
            where ad_object_id = '{act_id}'
            order by created_at desc
      """
        logger.debug(sql_statement)
        try:
            results = self._get_connection().execute(sql_statement)
            access_tokens = [result[0] for result in results]
            if len(access_tokens) > 1:
                logger.warning("channel: {channel}, id: {id} has more than 1 secrets in the database")
            access_token = access_tokens[0]
            return access_token
        except Exception as e:
            logger.exception(f"Problem get {channel} credential for account: {id}")
            raise e

    def _adwords_get_refresh_token(self, id):
        channel = "adwords"
        sql_statement = f"""
            -- choose the latest if multiple apply
            SELECT refresh_token
            FROM {channel}_integrations
            where customer_id = '{id}'
            order by created_at desc
      """
        print(sql_statement)
        try:
            results = self._get_connection().execute(sql_statement)
            refresh_tokens = [result[0] for result in results]
            if len(refresh_tokens) > 1:
                logging.warning("channel: {channel}, id: {id} has more than 1 secrets in the database")
                refresh_token = refresh_tokens[0]
            return refresh_token
        except Exception as e:
            logger.exception(f"Problem get {channel} credential for account: {id}")
            raise e

    def _shopify_get_access_token(self, id):
        channel = 'shopify'
        sql_statement = f"""
            -- choose the latest if multiple apply
            SELECT access_token
            FROM {channel}_integrations
            where shop_name = '{id}'
            order by created_at desc
      """
        print(sql_statement)
        try:
            results = self._get_connection().execute(sql_statement)
            tokens = [result[0] for result in results]
            if len(tokens) > 1:
                logging.warning("channel: {channel}, id: {id} has more than 1 secrets in the database")
                token = tokens[0]
            return token
        except Exception as e:
            logger.exception(f"Problem get {channel} credential for account: {id}")
            raise e


if __name__ == '__main__':
    host = 'localhost'
    port = '5432'
    username = 'postgres'
    password = 'postgres'
    database = 'postgres'
    secrets_manager = SecretsManager(host=host, port=port, username=username, password=password, database=database)
    print(secrets_manager.get_facebook_act_ids())
    print([secrets_manager.get_facebook_access_token(act_id) for act_id in secrets_manager.get_facebook_act_ids()])

# @staticmethod
# def _facebook_get_client_id(client, environment):
#     return get_value_from_ssm(f"/rubix/insights/{environment}/secrets_engine/facebook/CLIENT_ID", client)

# @staticmethod
# def _facebook_get_client_secret(client, environment):
#     return get_value_from_ssm(f"/rubix/insights/{environment}/secrets_engine/facebook/CLIENT_SECRET", client)

# @staticmethod
# def _adwords_get_client_id(client, environment):
#     return get_value_from_ssm(f"/rubix/insights/{environment}/secrets_engine/adwords/CLIENT_ID", client)

# @staticmethod
# def _adwords_get_client_secret(client, environment):
#     return get_value_from_ssm(f"/rubix/insights/{environment}/secrets_engine/adwords/CLIENT_SECRET", client)

# @staticmethod
# def _adwords_get_developer_token(client, environment):
#     return get_value_from_ssm(f"/rubix/insights/{environment}/secrets_engine/adwords/DEVELOPER_TOKEN", client)

# def get_credential(self, channel, id, environment):
#     """Get Credential."""
#     if channel == 'analytics':
#         credential = self._analytics_get_credential(id)
#     if channel == 'facebook':
#         credential = {}
#         credential['access_token'] = self._facebook_get_access_token(id)
#         credential['app_id'] = self._facebook_get_client_id(self.client, environment)
#         credential['app_secret'] = self._facebook_get_client_secret(self.client, environment)
#     if channel == 'adwords':
#         credential = {}
#         credential['client_id'] = self._adwords_get_client_id(self.client, environment)
#         credential['client_secret'] = self._adwords_get_client_secret(self.client, environment)
#         credential['developer_token'] = self._adwords_get_developer_token(self.client, environment)
#         credential['refresh_token'] = self._adwords_get_refresh_token(id)
#     if channel == 'shopify':
#         import shopify
#         credential = {}
#         shop_url = f"{id}.myshopify.com"
#         access_token = self._shopify_get_access_token(id)
#         session = shopify.Session(shop_url, SHOPIFY_API_VERSION, access_token)
#         credential['session'] = session
#     return credential

# @staticmethod
# def _analytics_get_client_id(client, environment):
#     return get_value_from_ssm(f"/rubix/insights/{environment}/secrets_engine/analytics/CLIENT_ID", client)

# @staticmethod
# def _analytics_get_client_secret(client, environment):
#     return get_value_from_ssm(f"/rubix/insights/{environment}/secrets_engine/analytics/CLIENT_SECRET", client)

# @classmethod
# def _analytics_get_access_token(cls, refresh_token):
#     client_id = cls._analytics_get_client_id(cls.client, "prd")
#     client_secret = cls._analytics_get_client_secret(cls.client, "prd")
#     grant_type = "refresh_token"
#     payload = {
#         'client_id': client_id,
#         'client_secret': client_secret,
#         'refresh_token': refresh_token,
#         'grant_type': grant_type
#     }

#     headers = {'content-type': 'application/x-www-form-urlencoded'}
#     # 'https://oauth2.googleapis.com/token',
#     response = requests.post(
#         cls.ANALYTICS_TOKEN_URI,
#         data=payload,
#         headers=headers
#     )
#     access_token = json.loads(response.text)['access_token']
#     return access_token

# def _analytics_get_credential(self, id: str) -> str:
#     # current take the latest strategy
#     # TODO:
#     channel = "analytics"
#     sql_statement = f"""
#       -- choose the latest if multiple apply
#       SELECT refresh_token
#       FROM {channel}_integrations
#       where account_id = '{id}'
#       order by created_at desc
#   """
#     print(sql_statement)
#     try:
#         results = self._get_connection().execute(sql_statement)
#         refresh_tokens = [result[0] for result in results]
#         if len(refresh_tokens) > 1:
#             logging.warning(f"channel: {channel}, id: {id} has more than 1 secrets in the database")
#         refresh_token = refresh_tokens[0]
#         access_token = self._analytics_get_access_token(refresh_token)
#         credential = google.oauth2.credentials.Credentials(
#             access_token,
#             refresh_token=refresh_token,
#             token_uri=self.ANALYTICS_TOKEN_URI,
#             client_id=self._analytics_get_client_id(self.client, "prd"),
#             client_secret=self._analytics_get_client_secret(self.client, "prd")
#         )
#         return credential
#     except Exception as e:
#         logging.exception(f"Problem get {channel} credential for account: {id}")
#         raise e


#   @staticmethod
#   def _shopify_get_client_id(client, environment):
#       """API Key."""
#       return get_value_from_ssm(f"/rubix/insights/{environment}/secrets_engine/shopify/CLIENT_ID", client)

#   @staticmethod
#   def _shopify_get_client_secret(client, environment):
#       return get_value_from_ssm(f"/rubix/insights/{environment}/secrets_engine/shopify/CLIENT_SECRET", client)
